import yaml
from typing import Dict, Any
import logging


def get_config(config_file_name: str) -> Dict[Any, Any]:
    """ Load the config file from the config directory, use the yaml library to return a dictionary configuration

    Args:
        config_file_name (str): the name of the config file, this is the absolute path as a string

    Returns:
        Dict[Any, Any]: The config file as a python dictionary - has keys for properties and values for corresponding settings
    """
    assert config_file_name is not None
    
    config_file_name

    with open(config_file_name) as fp:
        try:
            cfg = yaml.safe_load(fp)
        except Exception:
            logging.error("Could not write to file at: " +  config_file_name)
            logging.error("Please verify file exists and valid, then try again!")

    return cfg



from typing import Dict, Any
from pandas import DataFrame
from abc import ABC, abstractmethod

class CloudDataWriter(ABC):
   def __init__(self, conn:Dict[str, Any]) -> None: 
       """ Abstract base class encapsulating data writes to a cloud endpoint

       Args:
           conn (Dict[str, Any]): Connection information to connect to the endpoint
       """
       self._cfg = conn
       return



from typing import Dict, Any
from cloud_writers.cloud_data_writer import CloudDataWriter
from pandas import DataFrame
import logging
from minio import Minio


REQUIRED_INFO = ["HOST_URL", "ACCESS_KEY", "SECRET_KEY"]

class MinioWriter(CloudDataWriter):

    def __init__(self, conn: Dict[str, Any]) -> None:
        """ This class abstracts the details of connecting and writing a data file to a Minio (S3) bucket.

        Args:
            conn (Dict[str, Any]): The connection information to connect to the bucket, please see REQUIRED_INFO for the list of parameters that must be provided
        """
        super().__init__(conn)
        return
    
    def write_data(self, bucket_name: str, source_file: str, destination_file: str) -> None:
        """ This method writes the source file argument to the bucket name argument

        Args:
            bucket_name (str): The bucket name to which the source file must be written
            source_file (str): The source file to be written to the remote location
            destination_file (str): The name of the file in the remote location, can be different from the source file
        """
        if self._cfg is None:
            logging.error("Initialize the reader with a connection configuration and try again")
        for req_key in REQUIRED_INFO:
            if not req_key in self._cfg:
                logging.error("{required_key} is not in supplied\
                             connection configuration, please add\
                             this and try connectiong again".format(required_key=req_key))
        
        HOST_URL = self._cfg["HOST_URL"]
        ACCESS_KEY = self._cfg["ACCESS_KEY"]
        SECRET_KEY = self._cfg["SECRET_KEY"]
        df : DataFrame = None

        try:
            client = Minio(endpoint=HOST_URL, access_key=ACCESS_KEY, secret_key=SECRET_KEY, secure=True)
            # The destination bucket and filename on the MinIO server
            # Make the bucket if it doesn't exist.
            found = client.bucket_exists(bucket_name)
            if not found:
                client.make_bucket(bucket_name)
                logging.info("Created bucket", bucket_name)
            else:
                logging.info("Bucket", bucket_name, "already exists")
                # Upload the file, renaming it in the process
                client.fput_object(bucket_name, destination_file, source_file)
                logging.info( source_file, "successfully uploaded as object", destination_file, "to bucket", bucket_name)

        except :
            logging.error("There was a problem connecting to your remote bucket. Check connection parameters and try again!")
        
        return 
        


from prefect import flow, task
import pandas as pd



@task(log_prints=True)
def load_data_file(dataset_df:pd.DataFrame):
    print(f"Loaded dataset")
    return dataset_df

@task(log_prints=True)
def apply_noise_filter_1(dataset_df:pd.DataFrame):
    valid_cust = ~dataset_df["Customer ID"].isna()
    valid_desc = ~dataset_df["Description"].isna()
    valid_stock_code = ~dataset_df["Description"].isna()
    noise_filter_1 = valid_cust & valid_desc
    dataset_df = dataset_df[noise_filter_1].reset_index(drop=True)
    return dataset_df

def good_quantity_record_check(x):
    try:
        f_x = float(x)
        if f_x > 0:
            return True
        else:
            return False # returned purchase
    except :
        return False

@task(log_prints=True)
def apply_noise_filter_2(dataset_df: pd.DataFrame):
    good_quantity_records = dataset_df["Quantity"].apply(good_quantity_record_check)
    dataset_df = dataset_df[good_quantity_records]
    dataset_df = dataset_df.reset_index(drop=True)
    return dataset_df

@task(log_prints=True)
def apply_noise_filter_3(dataset_df: pd.DataFrame):
    return_or_bank_charges = (dataset_df.Price == "BANK CHARGES") | (dataset_df.Price == "ADJUST")
    valid_purchases = ~ return_or_bank_charges
    dataset_df = dataset_df[valid_purchases]
    dataset_df = dataset_df.reset_index(drop=True)
    return dataset_df

def not_test_product(x):
    if "TEST" in x:
        return False
    elif x in ["ADJUST", "BANK CHARGES", "C2", "M"]:
        return False
    else:
        return True

@task(log_prints=True)
def apply_noise_filter_4(dataset_df: pd.DataFrame):
    valid_products = dataset_df["StockCode"].apply(not_test_product)
    dataset_df = dataset_df[valid_products]
    dataset_df = dataset_df.reset_index(drop=True)
    return dataset_df

@task(log_prints=True)
def transform_1(dataset_df: pd.DataFrame):
    attr_types = {"Invoice": str, "StockCode": str, "Description": str,\
             "Quantity": float, "InvoiceDate": 'datetime64[ns]', "Price": float,\
             "Customer ID": str, "Country": str}
    dataset_df = dataset_df.astype(attr_types)
    Q1_2010 = (dataset_df["InvoiceDate"].dt.year == 2010) & (dataset_df["InvoiceDate"].dt.quarter == 1)
    dataset_df = dataset_df[Q1_2010]
    dataset_df = dataset_df.reset_index(drop=True)
    return dataset_df

@task(log_prints=True)
def transform_2(dataset_df: pd.DataFrame):
    dataset_df["item_total"] = dataset_df["Quantity"] * dataset_df["Price"]
    dsbysc = dataset_df.groupby([dataset_df.InvoiceDate.dt.day_of_year, dataset_df.StockCode])
    dsbysc = dsbysc["item_total"].sum().to_frame().reset_index()
    dfQ1_PA = dsbysc.pivot(index="InvoiceDate", columns="StockCode", values="item_total").fillna(0)
    
    return dfQ1_PA


@flow(name="retail_pipleline")
def retail_pipeline():
    DATA_DIR = "../../data"
    fp1 = DATA_DIR + "/" + "online_retail_II.csv"
    fp2 = DATA_DIR + "/" + "retail_q1_sales_2010_summary.csv"
    plist = [load_data_file, apply_noise_filter_1,\
             apply_noise_filter_2, apply_noise_filter_3,\
             apply_noise_filter_4]
    df = pd.read_csv(fp1)
    print("Running processing steps...")
    for f in plist:
        df = f(df)
    tlist = [transform_1, transform_2]
    print("Running transform steps...")
    for f in tlist:
        df = f(df)
    print("Done data prep!. Writing results to disk...")
    df.to_csv(fp2, index=False)

    return
    


# run the flow!
if __name__=="__main__":
    retail_pipeline()


from pathlib import Path
import os
from importlib_resources import files

def get_ontology_path() ->Path:
    """ Return the ontology file path as a Path

    Returns:
        Path: the path to the ontology file 
    """
    file_path = str(files('kmds.ontology').joinpath('kmds_ontology.xml'))
    #p = os.path.join(Path(__file__).absolute().parent.parent, "ontology/kmds_ontology.xml")
    
    return file_path

def get_kb_file_path(file_name: str) ->Path:
    """ Given a file name for a knowledge base, return the path to it as a Path

    Args:
        file_name (str): the filename of the knowledge base

    Returns:
        Path: the path corresponding to the provided file name
    """

    file_path =  str(files('kmds.examples').joinpath(file_name))
    return file_path

def get_kb_dir() ->Path:
    return str(files('kmds.examples'))

def check_valid_path(file_path: Path) -> bool:
    """ Check if a file exists at the provided path

    Args:
        file_path (Path): The path to a file

    Returns:
        bool: True if the file exists, else False
    """
    if file_path.is_file():
        return True
    else:
        return False
    
def get_package_kb_path(kb_name: str) ->str:
    """ Return path string to a package knowledge base

    Args:
        kb_name (str): The knowledge base name

    Returns:
        str: The path (as a string) to the package knowledge base
    """
    file_path =  str(files('kmds.examples').joinpath(kb_name))
    return file_path

    







from owlready2 import *
from enum import Enum
from kmds.tagging import *
from kmds.utils.path_utils import get_ontology_path

import logging
LOG_FILE = "../../kmds.log"
logging.basicConfig(filename=LOG_FILE, format='%(filename)s: %(message)s',
                    level=logging.DEBUG)



class PipelineType(str, Enum):
    """ Enumeration to define the types of workflows that can be created

    Args:
        str (_type_): The enumeration values are strings
        Enum (_type_): The enumeration values are strings
    """
    KNOWLEDGE_APPLICATION_WORKFLOW = "Knowledge Application Workflow"
    KNOWLEDGE_EXTRACTION_EXPERIMENT_WORKFLOW = "Knowledge Extraction Experiment Workflow"

onto = get_ontology(get_ontology_path()).load()

def set_ontology(the_onto : Ontology) ->None:
    global onto
    onto = the_onto
    return
    

with onto:
    class Workflow(Thing):
        """ Ontology definition for workflow

        Args:
            Thing (_type_): Thing is a base concept from OWL
        """
        pass
    class KnowledgeApplicationWorkflow(Workflow):
        """ knowledge application workflow definition - inherits from workflow

        Args:
            Workflow (_type_): Base workflow classxs
        """

        pass
    
    class KnowledgeExtractionExperimentationWorkflow(Workflow):
        """ knowledge extraction experiment workflow definition - inherits from workflow

        Args:
            Workflow (_type_): Base workflow class
        """

        pass
    
    class description(DataProperty, FunctionalProperty):
        """ Description as property of workflow

        Args:
            DataProperty (_type_): This is a OWLReady2 way to say that this property is of a built in type as opposed to a defined type
            FunctionalProperty (_type_): This is a OWLReady2 way to say that this property is of a built in type as opposed to a defined type
        """
        

        domain = [Workflow]
        range = [str]

    class name(DataProperty, FunctionalProperty):
        """ name as property of workflow

        Args:
            DataProperty (_type_): This is a OWLReady2 way to say that this property is of a built in type as opposed to a defined type
            FunctionalProperty (_type_): This is a OWLReady2 way to say that this property is of a built in type as opposed to a defined type
        """

        domain = [Workflow]
        range = [str]

    class KMObservation(Thing):
        """ Base class for observations, the specific types of observations derive from this class

        Args:
            Thing (_type_): OWL base class for everything
        """
        pass
    
    class ExploratoryObservation(KMObservation):
        """ Class to capture exploratory observations

        Args:
            KMObservation (_type_): the base observation class
        """
        pass

    class has_exploratory_observations(ObjectProperty):
        """ property to capture the exploratory observations of the workflow

        Args:
            ObjectProperty (_type_): captures the exploratory observations of the workflow
        """
        domain = [Workflow]
        range = [ExploratoryObservation]

    class exploratory_observation_type(DataProperty, FunctionalProperty):
        """ property to capture the type of exploratory observation - see tagging types for the options

        Args:
            DataProperty (_type_): OWL2library specific way of defining a property with a built in value
            FunctionalProperty (_type_): OWL2library specific way of defining a property with a built in value
        """
        domain = [ExploratoryObservation]
        range = [str]



    class DataRepresentationObservation(KMObservation):
        """ property to capture the data representation observation

        Args:
            KMObservation (_type_): base class for observations
        """
        pass

    class has_data_representation_observations(ObjectProperty):
        """ Property to capture the data representation observations of a workflow

        Args:
            ObjectProperty (_type_): OWLReady2 specific way of capturing object properties
        """
        domain = [Workflow]
        range = [DataRepresentationObservation]

    class data_representation_observation_type(DataProperty, FunctionalProperty):
        """ Specify the data representation observation type

        Args:
            DataProperty (_type_): OWLReady2 specific way of specifying the data property type
            FunctionalProperty (_type_): OWLReady2 specific way of specifying the data property type
        """
        domain = [DataRepresentationObservation]
        range = [str]

    class ModellingChoiceObservation(KMObservation):
        """ Definition of modelling choice observations

        Args:
            KMObservation (_type_): Base class for observations
        """
        pass

    class has_modeling_choice_observations(ObjectProperty):
        """ Specify that the workflow has modelling choice observations

        Args:
            ObjectProperty (_type_): the OWLReady2 way to specify that these observations are objects
        """
        domain = [Workflow]
        range = [ModellingChoiceObservation]

    class modelling_choice_observation_type(DataProperty, FunctionalProperty):
        """ Specify that the modelling observation type in tagging modelling choice observations - see tagging for the options available

        Args:
            DataProperty (_type_): OWLReady2 specific way of describing properties
        """
        domain = [ModellingChoiceObservation]
        range = [str]

    class ModelSelectionObservation(Thing):
        """ Definition of Model Selection Observations

        Args:
            Thing (_type_): Base class for everything in OWL
        """
        pass
    
    class has_model_selection_observations(ObjectProperty):
        """ Specify that the workflow has model selection observations recorded

        Args:
            ObjectProperty (_type_): OWLReady2 specific way of capturing object properties
        """
        domain = [Workflow]
        range = [ModelSelectionObservation]

    class model_selection_observation_type(DataProperty, FunctionalProperty):
        """ Specify the model selection observation type - see tagging for options

        Args:
            DataProperty (_type_): OWLReady2 specific of specifying properties of builtin type
            FunctionalProperty (_type_): OWLReady2 specific of specifying properties of builtin type
        """
        domain = [ModelSelectionObservation]
        range = [str]

    class depends_on(ObjectProperty):
        """ Property to capture the workflows that a particular workflow is informed by, and, therefore depends on, these workflows must have been run prior to this workflow

        Args:
            ObjectProperty (_type_): OWLReady2 specific way of specifying Object properties, as opposed to built in types
        """
        domain = [Workflow]
        range = [KnowledgeExtractionExperimentationWorkflow]

    class ExperimentalObservation(Thing):
        """ Define experimental observation type

        Args:
            Thing (_type_): OWL base class for everything
        """
        pass

    class has_experimental_observations(ObjectProperty):
        """ Property of the workflow to capture its experimental observations

        Args:
            ObjectProperty (_type_): OWLReady2 specific way of specifying that this property is an object type as opposed to a built in type
        """
        domain = [Workflow]
        range = [ExperimentalObservation]
    
    class experimental_observation_type(DataProperty, FunctionalProperty):
        """ Property to capture the experimental observation type - see tagging for definition

        Args:
            DataProperty (_type_): OWLReady2 specific way of defining built in types as properties
            FunctionalProperty (_type_): OWLReady2 specific way of defining built in types as properties
        """
        domain = [ExperimentalObservation]
        range = [str]

    class PerformanceMeasure(Thing):
        """ Definition of performance measure as a concept

        Args:
            Thing (_type_): OWL base class for everything
        """
        pass

    class description(DataProperty):
        """ description of the workflow

        Args:
            DataProperty (_type_): OWLReady2 specific way of describing built in type properties
        """
        range = [str]


    class business_performance_measure(ObjectProperty):
        """ Capture the business performance measure of the workflow

        Args:
            ObjectProperty (_type_): OWLReady2 specific way of specifying object properties
        """
        domain = [KnowledgeApplicationWorkflow]
        range = [PerformanceMeasure]

    class experiment_performance_measure(ObjectProperty):
        """ Property to capture experiment performance measure

        Args:
            ObjectProperty (_type_): Experiment performance measure as an object property
        """
        domain = [KnowledgeExtractionExperimentationWorkflow]
        range = [PerformanceMeasure]

    class finding(DataProperty, FunctionalProperty):
        """ Speficy the finding property of an observation

        Args:
            DataProperty (_type_): OWLReady 2 specific way to say that the range is a srting
            FunctionalProperty (_type_): OWLReady 2 specific way to say that the domain is an object
        """
        Domain =[KMObservation]
        range = [str]

    class finding_sequence(DataProperty, FunctionalProperty):
        """ Speficy the finding sequence property of an observation

        Args:
            DataProperty (_type_): OWLReady 2 specific way to say that the range is an integer
            FunctionalProperty (_type_): OWLReady 2 specific way to say that the domain is an object
        """
        Domain =[KMObservation]   
        range = [int]


from typing import Dict, Any
from pandas import DataFrame
from abc import ABC, abstractmethod

class CloudDataReader(ABC):

   def __init__(self, conn:Dict[str, Any]) -> None: 
       """ This is the abstract base class for the cloud data readers.

       Args:
           conn (Dict[str, Any]):  A dictionary containing the connection parameters to connect to the data source (for example, an S3 bucket or a box account.)
       """
       self._cfg = conn
       return
    




from enum import Enum

class ExploratoryTags(str, Enum):
    """ Tags to capture the exploratory observation type

    Args:
        str (_type_): string type tag
        Enum (_type_): string type tag
    """

    RELEVANCE_OBSERVATION = "Relevance Observation" 
    DATA_QUALITY_OBSERVATION = "Data Quality Observation"

class DataRepresentationTags(str, Enum):
    """ Tags to capture the data representation observation type
    Args:
        str (_type_): string type tag
        Enum (_type_): string type tag
    """

    FEATURE_ENGG_OBSERVATION = "Feature Engineering Observation"
    DATA_TRANSFORMATION_OBSERVATION = "Data Transformation Observation"

class ModellingChoiceTags(str, Enum):
    """ Tags corresponding to Modelling choice observation type
    Args:
        str (_type_): string type tag
        Enum (_type_): string type tag
    """

    MODELLING_CHOICE_OBSERVATION = "Modelling Choice Observation"
    MODELLING_ASSUMPTION_OBSERVATION = "Modelling Assumption Observation"

class ExperimentationTags(str, Enum):
    """ Tags corresponding to experimentation observation type
    Args:
        str (_type_): string type tag
        Enum (_type_): string type tag
    """

    HYPOTHESIS_STATEMENT = "Hypothesis Statement"
    EXPERIMENTAL_OBSERVATION = "Experimental Observation"
    EXPERIMENTAL_CONJECTURE = "Experimental Conjecture"
    RESULT_SUMMARY = "Experimental Result Summary"

class ModelSelectionTags(str, Enum):
    """ Tags corresponding to Model Selection observation type
    Args:
        str (_type_): string type tag
        Enum (_type_): string type tag
    """
    
    MODEL_SELECTION_STATEMENT = "Model Selection Statement"
    MODEL_SELECTION_SETUP_DESCRIPTION = "Model Selection Setup Description"
    MODEL_SELECTION_OBSERVATION = "Model Selection Observation"
    MODEL_SELECTION_RESULT_SUMMARY = "Model Selection Result Summary"




from typing import Dict, Any
from cloud_readers.cloud_data_reader import CloudDataReader
from pandas import DataFrame
import logging
from minio import Minio
import pandas as pd
from io import StringIO

REQUIRED_INFO = ["HOST_URL", "ACCESS_KEY", "SECRET_KEY"]

class MinioReader(CloudDataReader):

    def __init__(self, conn: Dict[str, Any]) -> None:
        """ This is the class to read data from a Minio cloud storage bucket. 

        Args:
            conn (Dict[str, Any]): This is the connection information to connect and read data from the Minio storage location. Please see the example notebook provided for details that are needed. The information in the REQUIRED_INFO list must be provided. There is a utility to convert a yaml file containing this information to python dictionary. Please see the example notebook for details.
        """
        super().__init__(conn)
        return
    
    def read_data(self, bucket_name: str, object_name: str) -> DataFrame:
        """ Use this method to read a object (data file) from the bucket (bucket name)

        Args:
            bucket_name (str): the bucket name containing the datafile
            object_name (str): data file with bucket that must be read

        Returns:
            DataFrame: A pandas datafreame wrapping the data file
        """
        if self._cfg is None:
            logging.error("Initialize the reader with a connection configuration and try again")
        for req_key in REQUIRED_INFO:
            if not req_key in self._cfg:
                logging.error("{required_key} is not in supplied\
                             connection configuration, please add\
                             this and try connectiong again".format(required_key=req_key))
        
        HOST_URL = self._cfg["HOST_URL"]
        ACCESS_KEY = self._cfg["ACCESS_KEY"]
        SECRET_KEY = self._cfg["SECRET_KEY"]
        df : DataFrame = None

        try:
            client = Minio(endpoint=HOST_URL, access_key=ACCESS_KEY, secret_key=SECRET_KEY, secure=True)
            obj = client.get_object(bucket_name, object_name)
            data = obj.data
            s = str(data,'utf-8')
            data = StringIO(s)
            df = pd.read_csv(data)
        except :
            msg = "Check connection parameters and if the object to be read exists in the remote bucket. There was a problem getting the remote object"
            logging.error(msg)
        
        return df
        
            
        


        



from kmds.ontology.kmds_ontology import *
from owlready2.namespace import Ontology
from typing import List
from pandas import DataFrame
import logging



def get_workflow(onto: Ontology) -> Workflow:
    """Given an ontology, return the workflow instance

    Args:
        onto (Ontology): The ontology from which the workflow instance must be loaded

    Returns:
        Workflow: The workflow in this ontology
    """
    the_workflow_instance = None
    
    with onto:
        workflow_instances = Workflow.instances()
        if workflow_instances is None:
            print("No workflow available")

        else:
            if len(workflow_instances) > 1:
                logging.info("There was more than one workflow, using the first one")
            
            the_workflow_instance = workflow_instances[0]

    return the_workflow_instance


def load_kb(kb_location: str) -> Ontology:
    """ Load knowledge base. The user can verify the knowledge base path with check_valid_path() in
    path utils, the knowledge base can be a url, the onus of verifying that that the knowledgebase location is valid is on the caller of the method, this method does not verify the validity

    Args:
        kb_name (str): the knowledge base to load

        Returns:
            Ontology: Returns a previously created knowledge base
    """
    onto :Ontology = None
    try:
        onto = get_ontology(kb_location).load()
        set_ontology(onto)
    except Exception as e:
        logging.error("Error opening KB, check if KB exists and permissions are right")
    if onto is None:
        logging.error("Could not load Ontology, check the file and try again.")


    return onto


def load_exp_observations(onto: Ontology) -> List[ExploratoryObservation]:
    """ Return the exploratory data analysis observations from the ontology

    Args:
        onto (Ontology): The ontology from which the EDA observations are to be returned

    Returns:
        List[ExploratoryObservation]: List of EDA observations in the ontology
    """


    the_workflow: Workflow = get_workflow(onto)
    exp_obs: List[ExploratoryObservation] = the_workflow.has_exploratory_observations
    records = []

    for o in exp_obs:
        a_row = {}
        a_row["obs_type"] = o.exploratory_observation_type
        a_row["finding"] = o.finding
        a_row["finding_seq"] = o.finding_sequence
        records.append(a_row)

    return DataFrame(records)


def load_data_rep_observations(onto: Ontology) -> List[DataRepresentationObservation]:
    """ Load the data representation observations from the provided ontology

    Args:
        onto (Ontology): The ontology containing the data representation observations

    Returns:
        List[DataRepresentationObservation]: The list of data representation observations
    """


    the_workflow: Workflow = get_workflow(onto)
    dr_obs: List[DataRepresentationObservation] = the_workflow.has_data_representation_observations
    records = []

    for o in dr_obs:
        a_row = {}
        a_row["obs_type"] = o.data_representation_observation_type
        a_row["finding"] = o.finding
        a_row["finding_seq"] = o.finding_sequence
        records.append(a_row)

    return DataFrame(records)

def load_modelling_choice_observations(onto: Ontology) -> List[ModellingChoiceObservation]:
    """ Load the modelling choice observations from the ontology

    Args:
        onto (Ontology): The ontology containing modelling choice observations

    Returns:
        List[ModellingChoiceObservation]: The list of modelling choice observations in the ontology
    """


    the_workflow: Workflow = get_workflow(onto)
    mc_obs: List[ModellingChoiceObservation] = the_workflow.has_modeling_choice_observations
    records = []

    for o in mc_obs:
        a_row = {}
        a_row["obs_type"] = o.modelling_choice_observation_type
        a_row["finding"] = o.finding
        a_row["finding_seq"] = o.finding_sequence
        records.append(a_row)

    return DataFrame(records)

def load_model_selection_observations(onto: Ontology) -> List[ModelSelectionObservation]:
    """ Load the model selection observations from the ontology

    Args:
        onto (Ontology): The ontology containing the model selection observations

    Returns:
        List[ModelSelectionObservation]: The list of model selection observations
    """


    the_workflow: Workflow = get_workflow(onto)
    mc_obs: List[ModelSelectionObservation] = the_workflow.has_model_selection_observations
    records = []

    for o in mc_obs:
        a_row = {}
        a_row["obs_type"] = o.model_selection_observation_type
        a_row["finding"] = o.finding
        a_row["finding_seq"] = o.finding_sequence
        records.append(a_row)

    return DataFrame(records)



from typing import Dict, Any
from cloud_readers.cloud_data_reader import CloudDataReader
from pandas import DataFrame
import logging
from minio import Minio
import pandas as pd
from io import StringIO
from boxsdk import OAuth2, Client

REQUIRED_INFO = ["CLIENT_ID", "CLIENT_SECRET", "ACCESS_TOKEN"]

class BoxReader(CloudDataReader):


    def __init__(self, conn: Dict[str, Any]) -> None:
        """ This is the class that abstracts the functionality to read from a dropbox collection. To use this reader, create a yaml file (see the repo for an example) with the required entries (see REQUIRED_INFO) and then call the read method with the file identifier of the data file you want to read. Please see the box reader example in the examples directory for details. This file identifier hanlde is a box specific quirk. You can find it by inspecting the file when you log into box. To do: I will subsequently find a way to get the file id from the name. Sorry for the incovinience.

        Args:
            conn (Dict[str, Any]): The information needed to connect to your box account. Create a yaml file with the required entries and then use the config file reader utility to convert to a python dict. Please see the Box reader example for details of how to use this
        """
        super().__init__(conn)
        return

    
    def read_data(self, file_id:str) -> DataFrame:
        """ Use this method to read the data file as a pandas data frame from the source box account. You will need to know the file identifier of the resource within Box (this is a box quirk) to use this method

        Args:
            file_id (str): The file identifier of the data file in your box account. You can find this from the address bar on your browser when you browse the file on your box account

        Returns:
            DataFrame: _description_
        """
        if self._cfg is None:
            logging.error("Initialize the reader with a connection configuration and try again")
        for req_key in REQUIRED_INFO:
            if not req_key in self._cfg:
                logging.error("{required_key} is not in supplied\
                             connection configuration, please add\
                             this and try connectiong again".format(required_key=req_key))
        
        CLIENT_ID = self._cfg["CLIENT_ID"]
        CLIENT_SECRET = self._cfg["CLIENT_SECRET"]
        ACCESS_TOKEN = self._cfg["ACCESS_TOKEN"]
        df : DataFrame = None
        try:
            auth = OAuth2(client_id = CLIENT_ID,
                          client_secret = CLIENT_SECRET,
                          access_token = ACCESS_TOKEN)
            client = Client(auth)
            file_content = client.file(file_id).content()
            s = str(file_content,'utf-8')
            data = StringIO(s)
            df = pd.read_csv(data)
        except :
            msg = "Check your connection parameters, especially the access token. There is a problem connecting. Also verify that the file identifier for the object you want to retrive. You will need to log into Box to verify and obtain this information."

            logging.error(msg)
        
        return df
            
